/* xh ver.03 playlist frontend */

#include "xh.h"

/* maximized or minimized window 0 1 */
guint mxz = 0;
guint miz = 0;

/* button clicks on control unit */
gboolean handle_button_press(GtkWidget *widget, GdkEventButton *event, 
                              gpointer data) {
    gint row = -1;
    if (widget == filelist) {
        GtkTreePath *path;
        if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(filelist), event->x,
             event->y, &path, NULL, NULL, NULL)) {
            gint *indices = gtk_tree_path_get_indices(path);
            row = indices[0];
            if (event->button == 1)
                act = indices[0];
            gtk_tree_path_free(path);
        }
    }
    if (event->button == 2) {
        if (widget == filelist && row >= 0) {
            if (row == last_played)
                stop_playing();
            delete_row(row);
            update_window();
            return TRUE;
        }
        if (widget == eventbox) {
            gtk_window_resize (GTK_WINDOW(window), 420, 220);
            gtk_rc_reparse_all();
            return TRUE;
        }
    }
    if (event->button == 3) {
    /* Right-click. */
        if (widget == filelist && row >= 0) {
            last_row_hit = row;
            gtk_widget_show(itemdel);
        } else {
            last_row_hit = -1;
            gtk_widget_hide(itemdel);
        }
        gtk_menu_popup(GTK_MENU(popupmenu), NULL, NULL, NULL, NULL, 
               event->button, event->time);
        return TRUE;
    } else if (event->button == 1 && widget == eventbox) {
        gtk_window_begin_move_drag (GTK_WINDOW (window),
              event->button, event->x_root, event->y_root, event->time);
        return TRUE;
    } else {
    /* Another button click; use the normal handlers. */
        return FALSE;
    }
}

/* window "state" events */
gboolean handle_state(GtkWidget *widget, GdkEventWindowState *event, 
                       gpointer data) {
    if (event->new_window_state == GDK_WINDOW_STATE_MAXIMIZED) {
        mxz = 1;
        miz = 0;
        return TRUE;
    } else if (event->new_window_state == GDK_WINDOW_STATE_ICONIFIED) {
        miz = 1;
        mxz = 0;
        return TRUE;
    } else {
        miz = 0;
        mxz = 0;
        return TRUE;
    }
}

gboolean handle_ebox_scroll(GtkWidget *widget, GdkEventScroll *event, 
                             gpointer data) {
    if (event->direction == GDK_SCROLL_UP) {
        if (mxz) {
            return FALSE;
        } else if (miz) {
            gtk_window_deiconify (GTK_WINDOW(window));
            return TRUE;
        } else {
            gtk_window_maximize (GTK_WINDOW(window));
            return TRUE;
        }
    } else if (event->direction == GDK_SCROLL_DOWN) {
        if (miz) {
            return FALSE;
        } else if (mxz) {
            gtk_window_unmaximize (GTK_WINDOW(window));
            return TRUE;
        } else {
            gtk_window_iconify (GTK_WINDOW(window));
            return TRUE;
        } 
    } else {
        return FALSE;
    }
}

gboolean handle_row_activated(GtkTreeView *w, GtkTreePath *path,
                          GtkTreeViewColumn *col, gpointer d) {
    gint row = -1;
    row = gtk_tree_path_get_indices(path)[0];
    if ( row != -1) {
        start_playing(row);
        return TRUE;
    } else {
        return FALSE;
    }
}

/* fileselect buttons */
void handle_fileselector_cancel(GtkButton *widget, gpointer data) {
    gtk_widget_destroy(filesel);
    filesel = NULL;
}

void handle_load_ok(GtkButton *widget, gpointer data) {
    const gchar *filename;
    gchar *buf;
    clear_playlist();
    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
    buf = g_strdup(filename);
    read_playlist(buf, 0);
    g_free(buf);
    gtk_widget_destroy(filesel);
    filesel = NULL;
}

void handle_save_ok(GtkButton *widget, gpointer data) {
    const gchar *n;
    n = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
    if (write_playlist(n))
        status(_("Unable to save playlist."));
    gtk_widget_destroy(filesel);
    filesel = NULL;
}

void handle_add_ok(GtkButton *widget, gpointer data) {
    add_file(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
    gtk_widget_destroy(filesel);
    filesel = NULL;
    update_window();
}

#ifdef HAVE_NFTW
void handle_add_dir_ok(GtkButton *widget, gpointer data) {
    add_directory(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
    gtk_widget_destroy(filesel);
    filesel = NULL;
    update_window();
}
#endif

/* "Next" button */
void handle_next(GtkButton *widget, gpointer data) {
    click_next();
}

/* "Previous" button */
void handle_prev(GtkButton *widget, gpointer data) {
    pick_prev();
}

void set_mini() {
    if (mode_mini) {
        gtk_window_get_size(GTK_WINDOW(window), &attribute_width,
                               &attribute_height);
        gtk_widget_hide(listbox);
        gtk_window_resize(GTK_WINDOW(window), attribute_width, 1);
        gtk_widget_hide(imlisth);
        gtk_widget_show(imlists);
    } else {
        gtk_widget_show(listbox);
        sync_list();
        gtk_window_resize(GTK_WINDOW(window), attribute_width, 
                               attribute_height);
        gtk_widget_hide(imlists);
        gtk_widget_show(imlisth);
    }
}

/* "Playlist" button */
void handle_minitoggle() {
    mode_mini = 0;
    if (gtk_widget_get_visible(listbox)) {
        mode_mini = 1;
    }
    set_mini();
}

/* "Random" checkbox */
void handle_randomtoggle(GtkToggleButton *widget, gpointer data) {
    mode_random = 0;
    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (ckm))) {
        mode_random = 1;
    }
}

/* "Loop" checkbox */
void handle_looptoggle(GtkToggleButton *widget, gpointer data) {
    mode_loop = 0;
    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (ckl))) {
        mode_loop = 1;
    }
}

/* "PlayPause" button */
void handle_pause(GtkButton *widget, gpointer data) {
    pause_playing();
    update_window();
}

/* "Stop" button */
void handle_stop(GtkButton *widget, gpointer data) {
    stop_playing();
    update_window();
}

void handle_menu_delete(GtkMenuItem *widget, gpointer data) {
    if (last_row_hit == last_played)
        stop_playing();
    delete_row(last_row_hit);
    update_window();
}

/* fileselect menu */
void handle_menu_load(GtkMenuItem *widget, gpointer data) {
    gchar name[STRBUFSIZE];
    get_playlist_dir(name, STRBUFSIZE);
    if (filesel)
        return;
    filesel = gtk_file_selection_new(_("Load playlist"));
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
    gtk_widget_set_name(filesel, "filesel");
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), name);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->ok_button),
             "clicked", (GtkSignalFunc) handle_load_ok, filesel);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->cancel_button),
             "clicked", (GtkSignalFunc) handle_fileselector_cancel, filesel);
    gtk_widget_show(filesel);
}

void handle_menu_save(GtkMenuItem *widget, gpointer data) {
    gchar name[STRBUFSIZE];
    get_playlist_dir(name, STRBUFSIZE);
    if (filesel)
        return;
    filesel = gtk_file_selection_new(_("Save playlist"));
    gtk_widget_set_name(filesel, "filesel");
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), name);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->ok_button),
             "clicked", (GtkSignalFunc) handle_save_ok, filesel);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->cancel_button),
             "clicked", (GtkSignalFunc) handle_fileselector_cancel, filesel);
    gtk_widget_show(filesel);
}

void handle_menu_add(GtkMenuItem *widget, gpointer data) {
    gchar name[STRBUFSIZE];
    get_data_dir(name, STRBUFSIZE);
    if (filesel)
        return;
    filesel = gtk_file_selection_new(_("Add song"));
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), name);
    gtk_widget_set_name(filesel, "filesel");
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->ok_button),
             "clicked", (GtkSignalFunc) handle_add_ok, filesel);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->cancel_button),
             "clicked", (GtkSignalFunc) handle_fileselector_cancel, filesel);
    gtk_widget_show(filesel);
}

#ifdef HAVE_NFTW
void handle_menu_add_dir(GtkMenuItem *widget, gpointer data) {
    gchar name[STRBUFSIZE];
    get_data_dir(name, STRBUFSIZE);
    if (filesel)
        return;
    filesel = gtk_file_selection_new(_("Add directory"));
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), name);
    gtk_widget_set_name(filesel, "filesel");
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->ok_button),
             "clicked", (GtkSignalFunc) handle_add_dir_ok, filesel);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel) ->cancel_button),
             "clicked", (GtkSignalFunc) handle_fileselector_cancel, filesel);
    gtk_widget_show(filesel);
}
#endif

/* Handlers for the sort menu items */
void handle_menu_sort_normal(GtkMenuItem *widget, gpointer data) {
    mode_sort_type = 0;
}

void handle_menu_sort_name(GtkMenuItem *widget, gpointer data) {
    sortplaylist(sort_compare_name, FALSE);
    mode_sort_type = 1;
}

void handle_menu_sort_swapped_name(GtkMenuItem *widget, gpointer data) {
    sortplaylist(sort_compare_swapped_name, FALSE);
    mode_sort_type = 2;
}

void handle_menu_sort_mtime(GtkMenuItem *widget, gpointer data) {
    sortplaylist(sort_compare_mtime, TRUE);
    mode_sort_type = 3;
}

void handle_menu_clear(GtkMenuItem *widget, gpointer data) {
    clear_playlist();
}

/* Fork a preferences window */
void handle_menu_prefs(GtkMenuItem *widget, gpointer data) {
    char go_prefs[32] = "/root/.xh/conf/setup";
    pid_t pid = fork ();
    switch (pid) {
        case 0:
            execlp ("/bin/sh", "sh", "-c", go_prefs, NULL);
            fprintf (stderr, "Exec failed.\n");
            exit (0);
            break;
        case -1:
            fprintf (stderr, "Fork failed.\n");
            break;
    }
}

void handle_menu_quit(GtkMenuItem *widget, gpointer data) {
    gtk_widget_destroy(GTK_WIDGET(window));
}

/* Handle the "destroy" event */
gboolean handle_destroy(GtkWidget *widget, GdkEvent *event, gpointer data) {
    /* Save the window position, if necessary. */
    if ((mode_window_xy) || (mode_window_wh))
        save_window_state();
    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (ckm))) {
        mode_random = 1;
    } else {
        mode_random = 0;
    }
    save_window_prop();
    /* Stop the player. */
    stop_playing();
    persist_playlist();
    /* Then quit the interface. */
    gtk_main_quit();
    return FALSE;
}

/* Handle "configure" events (moves & resizes) of the window */
gboolean handle_configure(GtkWidget *widget, GdkEventConfigure *event,
                           gpointer data) {
    gtk_window_get_position(GTK_WINDOW(widget), &attribute_xpos, &attribute_ypos);
    if (mode_mini) {
        gtk_window_get_size(GTK_WINDOW(widget), &attribute_width, &attribute_lheight);
    } else {
        gtk_window_get_size(GTK_WINDOW(widget), &attribute_width, &attribute_height);
    }
    return FALSE;
}

/* 1 if user defined or default playlist extension */
int xhlist(const char *file) {
    char *cfix;
    if (!file)
        return 0;
    if (!attribute_extlist)
        return 0;
    cfix = strrchr(file, '.');
    if (!cfix)
        return 0;
    cfix++;
    if (strncasecmp(cfix, attribute_extlist, 3) == 0 ||
        strncasecmp(cfix, "xhl", 3) == 0) {
        return 1;
    }
    return 0;
}

/* Handle files dropped on our window */
void handle_dnd_drop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
             GtkSelectionData *data, guint info, guint time, gpointer user_data) {
    if (data->data) {
        gchar *s = strdup((const char *) data->data), *p, *q;
        p = s;
        while ((q = strchr(p, '\r'))) {
            char *file;
            *q = '\0';
            if (!(*++q == '\n'))
                break;
            *q = '\0';
            /* accept locale specific filenames from ROX */
            file = g_filename_from_uri(p, NULL, NULL);
            if (file != NULL) {
                if (xhlist(file)) {
                    gchar *buff;
                    buff = g_strdup(file);
                    read_playlist(buff, 0);
                    g_free(buff);
                } else {
                    if (filetype(file) == 2) {
                        add_directory(file);
                    } else {
                        add_file(file);
                    }
                }
            }
            p = q + 1;
            g_free(file);
        }
        free(s);
    }
    update_window();
}

