/* xh ver.03 playlist frontend */

#include "xh.h"

/* Prototypes for static functions */
static void add_file_action(gchar *extension, gchar *action, gchar *options);
static void conf_read_playlist(gchar *name);
static void prefs_bool_callback(GtkWidget *widget, gpointer data);

/* Globals */
gboolean mode_play_on_start = 0,
    mode_scroll_catchup = 0,
    mode_loop = 0,
    mode_window_wh = 0,
    mode_window_xy = 0,
    mode_strip_extension = 0,
    mode_hide_player_output = 0,
    mode_demangle_names = 0,
    mode_playlist_title = 0,
    mode_title_basename = 0,
    mode_no_check_files = 0,
    mode_commandline_songs = 0,
    mode_commandline_dirs = 0,
    mode_delete_when_played = 0,
    mode_persist_playlist = 0,
    mode_commandline_guess = 0,
    mode_persist_start = 0,
    mode_mini = 0,
    mode_decorated = 0;
guint mode_skip_path = 0;
gchar *attribute_playlist_dir = NULL;
gchar *attribute_data_dir = NULL;
gchar *attribute_extlist = NULL;
gint mode_sort_type = 0,
    mode_random = 0;
gint attribute_xpos = 100, 
    attribute_ypos = 100, 
    attribute_width = 360,
    attribute_lheight = 325,
    attribute_height = 220;
fileaction *fileactions = NULL;

/* Configuration commands */
static configcommand *configcommands;
static int num_configcommands, size_configcommands;

/* Add a config command to the list */
static void add_confcmd(gint pass, configtype type, char *command,
                   void *target) {
    /* Grow the array if it's not big enough */
    if (num_configcommands >= size_configcommands) {
        size_configcommands += 50;
        configcommands = realloc(configcommands,
                 size_configcommands * sizeof *configcommands);
    }
    configcommands[num_configcommands].pass = pass;
    configcommands[num_configcommands].type = type;
    configcommands[num_configcommands].command = command;
    configcommands[num_configcommands].target = target;
    num_configcommands++;
}

/* Initialise the list of config commands */
void init_config_commands() {
    num_configcommands = 0;
    size_configcommands = 0;
    configcommands = NULL;
    add_confcmd(0, CT_BOOL, "xh00", &mode_play_on_start);
    add_confcmd(0, CT_BOOL, "xh01", &mode_scroll_catchup);
    add_confcmd(0, CT_BOOL, "xh02", &mode_window_wh);
    add_confcmd(0, CT_BOOL, "xh03", &mode_window_xy);
    add_confcmd(0, CT_BOOL, "xh04", &mode_strip_extension);
    add_confcmd(0, CT_BOOL, "xh05", &mode_hide_player_output);
    add_confcmd(0, CT_BOOL, "xh06", &mode_demangle_names);
    add_confcmd(0, CT_BOOL, "xh07", &mode_playlist_title);
    add_confcmd(0, CT_BOOL, "xh08", &mode_title_basename);
    add_confcmd(0, CT_BOOL, "xh09", &mode_no_check_files);
    add_confcmd(0, CT_BOOL, "xh10", &mode_delete_when_played);
    add_confcmd(0, CT_BOOL, "xh11", &mode_persist_playlist);
    add_confcmd(0, CT_BOOL, "xh12", &mode_persist_start);
    add_confcmd(0, CT_BOOL, "xh13", &mode_decorated);
    add_confcmd(0, CT_BOOL, "cmdsongs", &mode_commandline_songs);
    add_confcmd(0, CT_BOOL, "cmddirs", &mode_commandline_dirs);
    add_confcmd(0, CT_BOOL, "cmdguess", &mode_commandline_guess);
    add_confcmd(0, CT_INT, "xh14", &mode_skip_path);
    add_confcmd(0, CT_STRING, "listdir", &attribute_playlist_dir);
    add_confcmd(0, CT_STRING, "datadir", &attribute_data_dir);
    add_confcmd(0, CT_STRING, "listextension", &attribute_extlist);
    add_confcmd(0, CT_FUNCTION23, "type", add_file_action);
    add_confcmd(1, CT_FUNCTION1, "load", conf_read_playlist);
    add_confcmd(1, CT_FUNCTION1, "exec", system);
    add_confcmd(0, CT_END, NULL, NULL);
}

/* Read a playlist */
static void conf_read_playlist(gchar *name) {
    read_playlist(name, 0);
}

/* Add a file action to the list */
static void add_file_action(gchar *extension, gchar *action, gchar *options) {
    char *p;
    fileaction *fa = (fileaction *)malloc(sizeof(fileaction));
    fa->extension = strdup(extension);
    fa->action = strdup(action);
    /* Make fa->name contain the name up to the first space */
    fa->name = strdup(action);
    p = strchr(fa->name, ' ');
    if (p)
        *p = '\0';
    fa->setgroup = fa->nullinput = 0;
    if (options) {
        if (strchr(options, 'g'))
            fa->setgroup = 1;
        if (strchr(options, 'i'))
            fa->nullinput = 1;
    }
    fa->next = fileactions;
    fileactions = fa;
}

/* Execute a configuration command. This may have uses other than
   reading the config file in the future (perhaps for remote
   control) */
void execute_command(gchar *command, gint pass) {
    gchar *cmd = strdup(command), *opta, *optb = NULL, *optc = NULL;
    configcommand *cc;
    if (!*cmd)
        return;
    if (*cmd == '#')
        return;
    opta = strchr(cmd, ':');
    if (opta) {
        *opta++ = '\0';
        optb = strchr(opta, ':');
        if (optb) {
            *optb++ = '\0';
            optc = strchr(optb, ':');
            if (optc)
                *optc++ = '\0';
        }
    }
    for (cc = configcommands; cc->type != CT_END; cc++) {
        if (strcmp(cmd, cc->command) != 0)
            continue;
        if (cc->pass != pass)
            continue;
        switch (cc->type) {
            case CT_BOOL: {
                if (!opta) {
                    fprintf(stderr, "Command '%s' needs an argument.", cmd);
                    goto out;
                }
                if (strcmp(opta, "yes") == 0 || strcmp(opta, "true") == 0 
                              || atoi(opta) != 0) {
                    *(gboolean *)cc->target = 1;
                } else {
                    *(gboolean *)cc->target = 0;
                }
                break;
            }
            case CT_INT: {
                if (!opta) {
                    fprintf(stderr, "Command '%s' needs an argument.", cmd);
                    goto out;
                }
                *(guint *)cc->target = atoi(opta);
                break;
            }
            case CT_STRING: {
                if (!opta) {
                    fprintf(stderr, "Command '%s' needs an argument.", cmd);
                    goto out;
                }
                *(gchar **)cc->target = strdup(opta);
                break;
            }
            case CT_FUNCTION1: {
                void (*func)(gchar *a) = cc->target;
                if (!opta) {
                    fprintf(stderr, "Command '%s' needs an argument.", cmd);
                    goto out;
                }
                func(opta);
                break;
            }
            case CT_FUNCTION23: {
                void (*func)(gchar *a, gchar *b, gchar *c) = cc->target;
                if (!opta || !optb) {
                    fprintf(stderr, 
                            "Command '%s' needs at least two arguments.", cmd);
                    goto out;
                }
                func(opta, optb, optc);
                break;
            }
            case CT_END:
            /* This will not happen. */
                break;
        }
    /* And, since we've matched the command, there's no point
       in looking any further */
    break;
    }
    out:
        free(cmd);
}

/* Read a config file, named in name. Pass 0 is before the GUI is
   shown, pass 1 is after */
void read_config(gchar *name, gint pass) {
    FILE *f;
    gchar line[STRBUFSIZE];
    f = fopen(name, "r");
    if (!f)
        return;
    while (fgets(line, STRBUFSIZE, f)) {
        chomp(line);
        execute_command(line, pass);
    }
    fclose(f);
}

/* Read config files */
void read_config_file(gint pass) {
    gchar rcname[STRBUFSIZE];
    snprintf(rcname, STRBUFSIZE, "%s/.xh/options", getenv("HOME"));
    read_config(rcname, pass);
}

/* Get the directory in which the user wants to store playlists */
void get_playlist_dir(gchar *buf, int buflen) {
    /* If attribute_playlist_dir is set, use that */
    if (attribute_playlist_dir) {
        snprintf(buf, buflen, "%s", attribute_playlist_dir);
        return;
    }
    /* If $HOME/.xh/playlists is a directory, use that */
    snprintf(buf, buflen, "%s/.xh/playlists/", getenv("HOME"));
    if (filetype(buf) == 2)
        return;
    /* Else use nothing */
    strcpy(buf, "");
}

/* Get the user data directory, for add_file menu */
void get_data_dir(gchar *buf, int buflen) {
    /* If attribute_data_dir is set, use that */
    if (attribute_data_dir) {
        snprintf(buf, buflen, "%s", attribute_data_dir);
        return;
    }
    /* If $HOME/my-documents is a directory, use that as fallback option */
    snprintf(buf, buflen, "%s/my-documents/", getenv("HOME"));
    if (filetype(buf) == 2)
        return;
    /* Else use nothing */
    strcpy(buf, "");
}



/* Save the window position */
void save_window_state() {
    char filename[STRBUFSIZE];
    FILE *f;
    snprintf(filename, STRBUFSIZE, "%s/.xh/conf/aw", getenv("HOME"));
    if ((f = fopen(filename, "w"))) {
        fprintf(f, "%d\n%d\n%d\n%d\n%d\n%d\n", attribute_xpos, 
                 attribute_ypos, attribute_width, attribute_lheight,
                 attribute_height, mode_mini);
        fclose(f);
    } else {
        g_print(_("Couldn't write to configuration file.\n"));
    }
}

/* Read the window position */
void read_window_state() {
    char filename[STRBUFSIZE];
    char line[VALBUFSIZE];
    FILE *f;
    snprintf(filename, STRBUFSIZE, "%s/.xh/conf/aw", getenv("HOME"));
    if ((f = fopen(filename, "r"))) {
        if (fgets(line, VALBUFSIZE, f) != NULL)
            attribute_xpos = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            attribute_ypos = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            attribute_width = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            attribute_lheight = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            attribute_height = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            mode_mini = atoi(line);
        fclose(f);
    }
    /* else fail silently, it happens all the time */
}

/* Save menu option */
void save_window_prop() {
    char filename[STRBUFSIZE];
    FILE *f;
    snprintf(filename, STRBUFSIZE, "%s/.xh/conf/am", getenv("HOME"));
    if ((f = fopen(filename, "w"))) {
        fprintf(f, "%d\n%d\n%d\n", mode_sort_type, mode_random, mode_loop);
        fclose(f);
    } else {
        g_print(_("Couldn't write to configuration file.\n"));
    }
}

/* Read menu option */
void read_window_prop() {
    char filename[STRBUFSIZE];
    char line[VALBUFSIZE];
    FILE *f;
    snprintf(filename, STRBUFSIZE, "%s/.xh/conf/am", getenv("HOME"));
    if ((f = fopen(filename, "r"))) {
        if (fgets(line, VALBUFSIZE, f) != NULL)
            mode_sort_type = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            mode_random = atoi(line);
        if (fgets(line, VALBUFSIZE, f) != NULL)
            mode_loop = atoi(line);
        fclose(f);
    }
}

