/* xh ver.03 playlist frontend */

#ifndef _XH_H
#define _XH 1

/* autoconf definitions */
#include "config.h"

#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <signal.h>
#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <time.h>
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include <sys/stat.h>
#include <glib.h>
#ifndef NOGETOPTLONG
#include <getopt.h>
#endif
#ifdef HAVE_NFTW
#define __USE_XOPEN_EXTENDED 1
#include <ftw.h>
#undef __USE_XOPEN_EXTENDED
#endif
#include <locale.h>

/* This is necessary for MacOS X, where using the default redirection
   mechanism generates binaries with unresolved symbols due to some
   reasonably stupid kludges in gettext.
#define _INTL_REDIRECT_MACROS 1 */

#include <libintl.h>
#define _(X) gettext(X)

/* --- Constants --- */

/* A generic string buffer size used throughout */
#define STRBUFSIZE 1024
#define VALBUFSIZE 32

/* --- Types --- */

/* A file type linked list */
struct _fileaction {
    gchar *extension;        /* The extension of the file */
    gchar *action;           /* The command to run */
    gchar *name;             /* The name of the command */
    char setgroup;           /* Whether the player should be run in a
                                  separate process group */
    char nullinput;          /* Whether the player's stdin should be
                                  connected to /dev/null */
    struct _fileaction *next;
};
typedef struct _fileaction fileaction;

/* Information about a song in the playlist */
struct _songinfo {
    gchar *name;             /* Its filename */
    gchar *display_name;     /* The name shown in the list */
    char played;             /* Whether it has been played */
    char inforead;           /* Whether the information that comes 
                                  from stat (mtime) has been read */
    time_t mtime;            /* When the file was last modified */              
    int previous;            /* The index of the song played before this one
                                  when this song was last picked by the 
                                  randomiser. -1 if no previous song 
                                  was picked */
};
typedef struct _songinfo songinfo;

enum _configtype {
    CT_END,                   /* end of list */
    CT_BOOL,                  /* gboolean */
    CT_INT,                   /* guint */
    CT_STRING,                /* gchar * */
    CT_FUNCTION1,             /* void function(gchar *arg) */
    CT_FUNCTION23             /* void function(gchar *arg1, gchar *arg2,
                                 gchar *arg3) (arg3 can be NULL) */
};
typedef enum _configtype configtype;

struct _configcommand {
    gint pass;                /* Pass upon which to read this entry */
    configtype type;          /* Type of this command */
    const char *command;      /* Name of this command */
    void *target;
};
typedef struct _configcommand configcommand;

/* --- Globals --- */

/* UI widgets */
extern GtkWidget *window, *frameb, *allbox, *hnbox, *movebox, *eventbox, 
    *statusbar, *btbox, *btstop, *btprev, *btplay, *imbox, *implay, *impause, 
    *btnext, *btlist, *listbox, *popupmenu, *itemdel, *msep1, *msep2, *msep3, 
    *iteml, *subl, *ckm, *ckl, *itemr, *subr, *filesel, *filelist,
    *rb0, *rb1, *rb2, *rb3, *imbox2, *imlists, *imlisth;
extern GtkTreeViewColumn *column;
extern GtkListStore *filestore;

enum {
    SONGINFO_COLUMN,
    TITLE_COLUMN,
    NUM_COLUMNS
};

extern GtkAccelGroup *accel_group;
extern int sigchld_pipe[2];
extern gchar startup_cwd[STRBUFSIZE];

/* Status variables */
extern pid_t childpid;
extern guint contextid,
    listcount, 
    playing, 
    paused;
extern guint mxz,
    miz;
extern gint last_played,
    last_row_hit,
    act;
fileaction *last_action;

/* Configuration options */
extern gboolean mode_play_on_start,
    mode_scroll_catchup,
    mode_loop,
    mode_window_wh,
    mode_window_xy,
    mode_strip_extension,
    mode_hide_player_output,
    mode_demangle_names,
    mode_playlist_title,
    mode_title_basename,
    mode_no_check_files,
    mode_commandline_songs,
    mode_commandline_dirs,
    mode_delete_when_played,
    mode_persist_playlist,
    mode_commandline_guess,
    mode_persist_start,
    mode_mini,
    mode_decorated;
extern guint mode_skip_path;
extern gchar *attribute_playlist_dir;
extern gchar *attribute_data_dir;
extern gchar *attribute_extlist;
extern gint mode_sort_type,
    mode_random;
extern gint attribute_xpos, 
    attribute_ypos, 
    attribute_width, 
    attribute_lheight,
    attribute_height;
extern fileaction *fileactions;

/* --- Prototypes --- */
songinfo *copy_songinfo(songinfo *src);
void destroy_songinfo(gpointer sinfo);
void delete_row(gint location);
void insert_row(songinfo *sinfo, gint location);
songinfo *get_songinfo(gint number);
void persist_playlist(void);
gint filetype(char *file);
void chomp(gchar *line);
void status(const char *message);
void update_window(void);
void sync_list(void);
void stop_playing(void);
void pause_playing(void);
fileaction *get_action(const gchar *name);
void start_playing(gint num);
void pick_prev(void);
void pick_next(void);
void click_next(void);
void play_show();
void pause_show();
void read_song_info(songinfo *sinfo, struct stat *st);
void add_file(const gchar *name);
void add_directory(const gchar *name);
void save_window_state(void);
void read_window_state(void);
void save_window_prop(void);
void read_window_prop(void);
void clear_playlist(void);
int sort_compare_name(const void *a, const void *b);
int sort_compare_swapped_name(const void *a, const void *b);
int sort_compare_mtime(const void *a, const void *b);
void sortplaylist(int (*func)(const void *a, const void *b),
          char needinfo);
void read_playlist(gchar *name, int use_cwd);
int write_playlist(const gchar *name);
void init_config_commands();
void execute_command(gchar *command, gint pass);
void read_config(gchar *name, gint pass);
void read_config_file(gint pass);
gboolean handle_button_press(GtkWidget *widget, GdkEventButton *event,
                 gpointer data);
gboolean handle_state(GtkWidget *widget, GdkEventWindowState *event, 
                 gpointer data);
gboolean handle_ebox_scroll(GtkWidget *widget, GdkEventScroll *event, 
                 gpointer data);
gboolean handle_row_activated(GtkTreeView *w, GtkTreePath *path,
                 GtkTreeViewColumn *col, gpointer d);
void handle_row_inserted(GtkTreeModel *model, GtkTreePath *path,
                 GtkTreeIter *iter, gpointer d);
void handle_row_deleted(GtkTreeModel *model, GtkTreePath *path, gpointer d);
void handle_fileselector_cancel(GtkButton *widget, gpointer data);
void handle_load_ok(GtkButton *widget, gpointer data);
void handle_save_ok(GtkButton *widget, gpointer data);
void handle_add_ok(GtkButton *widget, gpointer data);
#ifdef HAVE_NFTW
void handle_add_dir_ok(GtkButton *widget, gpointer data);
#endif
void handle_next(GtkButton *widget, gpointer data);
void handle_prev(GtkButton *widget, gpointer data);
void set_mini(void);
void handle_minitoggle();
void handle_randomtoggle(GtkToggleButton *widget, gpointer data);
void handle_looptoggle(GtkToggleButton *widget, gpointer data);
void handle_restart(GtkButton *widget, gpointer data);
void handle_stop(GtkButton *widget, gpointer data);
void handle_pause(GtkButton *widget, gpointer data);
void handle_menu_delete(GtkMenuItem *widget, gpointer data);
void get_playlist_dir(gchar *buf, int buflen);
void get_data_dir(gchar *buf, int buflen);
void handle_menu_load(GtkMenuItem *widget, gpointer data);
void handle_menu_save(GtkMenuItem *widget, gpointer data);
void handle_menu_add(GtkMenuItem *widget, gpointer data);
#ifdef HAVE_NFTW
void handle_menu_add_dir(GtkMenuItem *widget, gpointer data);
#endif
void handle_menu_sort_normal(GtkMenuItem *widget, gpointer data);
void handle_menu_sort_name(GtkMenuItem *widget, gpointer data);
void handle_menu_sort_swapped_name(GtkMenuItem *widget, gpointer data);
void handle_menu_sort_mtime(GtkMenuItem *widget, gpointer data);
void handle_menu_clear(GtkMenuItem *widget, gpointer data);
void handle_menu_prefs(GtkMenuItem *widget, gpointer data);
void handle_menu_quit(GtkMenuItem *widget, gpointer data);
gboolean handle_destroy(GtkWidget *widget, GdkEvent *event, gpointer data);
gboolean handle_configure(GtkWidget *widget, GdkEventConfigure *event,
              gpointer data);
void sigchld_backend(gpointer data, gint source, GdkInputCondition cond);
void handle_sigchld(int dummy);
int xhlist(const char *file);
void handle_dnd_drop(GtkWidget *widget, GdkDragContext *context,
             gint x, gint y,
             GtkSelectionData *data, guint info,
             guint time, gpointer user_data);
int main(int argc, char **argv);

/* --- Compatibility stuff --- */

#ifndef HAVE_SNPRINTF
/* If the user doesn't have snprintf, use sprintf instead. 
   This is a GNU cpp extension. */
#define snprintf(s, l, f, a...) sprintf(s, f, ##a)
#endif

#ifdef NOSTRCASECMP
/* The system doesn't provide strcasecmp, so we will instead */
int strcasecmp(char *a, char *b);
#endif

#endif /* _XH_H */

